# params:
#
# period: time between rate update steps in s
set period 1.0
#
# step: the rate increase amount in bytes/s
set step [expr 3000.0 / 8.0]

#Create a simulator object
set ns [new Simulator]

#Open the output files

set f [open out.tr w]
$ns trace-all $f

#Create 5 nodes
set n0 [$ns node]
set n1 [$ns node]
set n2 [$ns node]
set n3 [$ns node]
set n4 [$ns node]
set n5 [$ns node]


#Connect the nodes
$ns duplex-link $n0 $n2 0.1Mb 10ms DropTail
$ns duplex-link-op $n0 $n2 orient right-down

$ns duplex-link $n1 $n2 0.1Mb 10ms DropTail
$ns duplex-link-op $n1 $n2 orient right-up

$ns duplex-link $n2 $n3 0.3Mb 10ms DropTail
$ns duplex-link-op $n2 $n3 orient right

$ns duplex-link $n3 $n4 0.1Mb 10ms DropTail
$ns duplex-link-op $n3 $n4 orient right-up

$ns duplex-link $n3 $n5 1Mb 10ms DropTail
$ns duplex-link-op $n3 $n5 orient right-down


# Create two CBR agents + Null sinks

set cbr0 [new Agent/CBR]
$cbr0 set packetSize_ 100
$cbr0 set interval_ [expr [$cbr0 set packetSize_] / 8000.0]
$ns attach-agent $n0 $cbr0

set cbr1 [new Agent/CBR]
$cbr1 set packetSize_ 100
$cbr1 set interval_ [expr [$cbr1 set packetSize_] / 8000.0]
$ns attach-agent $n1 $cbr1

set sink0 [new Agent/Null]
$ns attach-agent $n4 $sink0

set sink1 [new Agent/Null]
$ns attach-agent $n5 $sink1

$ns connect $cbr0 $sink0
$ns connect $cbr1 $sink1


#Define a 'finish' procedure
proc finish {} {
	global f
	#Close the output file
        close $f
      exit 0
}


#Define a procedure which periodically increases the rate of the CBR agents
proc increaseRate {} {
      global cbr0 cbr1 period step
 
    #Get an instance of the simulator
    set ns [Simulator instance]
    #Determine the current bandwidth
    set bw0 [expr [$cbr0 set packetSize_] / [$cbr0 set interval_]]
    set bw1 [expr [$cbr0 set packetSize_] / [$cbr0 set interval_]]
    #Update (increase by 1 step)
    set bw0 [expr $bw0 + $step]
    set bw1 [expr $bw1 + $step]
    #Install the new bandwidth
    $cbr0 set interval_ [expr [$cbr0 set packetSize_] / $bw0]
    $cbr1 set interval_ [expr [$cbr1 set packetSize_] / $bw1]
    
    #Get the current time
    set now [$ns now]
    #Re-schedule the procedure
    $ns at [expr $now+$period] "increaseRate"
}


#Start the traffic sources
$ns at 0.0 "$cbr0 start"
$ns at 0.0 "$cbr1 start"

#Start the rate update procedure
$ns at 0.0 "increaseRate"

#Call the finish procedure after 60 seconds simulation time
$ns at 60.0 "finish"

#Run the simulation
$ns run
